#include "p33FJ256MC710.h"
#include <math.h>
#include "uart.h"
#include "timer.h"
#include "Camera.h"


/****************************** Library Functions ********************************/
/* 	unsigned char Cam_InitialCmd(void):
	This function modifies the main global variable CAMSTATUS and reads the information 
	contained in the variable CAMCONFIG, initially modified by the user, to set the field 
	Cam_Cmd of data structure CamInit. In particular, the function can read information 
	about color, and resolution. CamInit is the data structure used by the function 
	Cam_SendCmd (), to prepare the transfer of the INITIAL command.
*/
unsigned char Cam_InitialCmd(void)
{ 
     unsigned char pictype, snaptype;    
     unsigned long int color, prevres, jpegres;
     CAMCMD CamInitial;

	 CAMSTATUS = CAMSTATUS & 0xFFFE; // CAMSTATUS Initialization

     pictype = (CAMCONFIG>>PICTYPE_FLAG) & PICTYPE_NUM;
     if( pictype==0 ) return 0;

     snaptype = (CAMCONFIG>>SNAPTYPE_FLAG) & SNAPTYPE_NUM;
     
     color = CAMCONFIG & COLOR_NUM;
     if(color == 0 || color == 4)  return 0;
     
     prevres = (CAMCONFIG>>PREVRES_FLAG) & PREVRES_NUM	;
     if(prevres == 0 || prevres == 2 || prevres == 4 || prevres == 6) return 0;
     
     jpegres = (CAMCONFIG>>JPEGRES_FLAG) & JPEGRES_NUM;
     if(jpegres == 0 || jpegres == 2 || jpegres == 4 || jpegres == 6) return 0;
   
     CamInitial.Id = CAM_CMD_INITIAL;
     CamInitial.Cam_Cmd = (color<<16) + (prevres<<8) + jpegres;
     
     return Cam_SendCmd(&CamInitial);
}

/* 	unsigned char Cam_AckCmd(unsigned long int ackcmd):
   	This function modifies the main global variable CAMSTATUS and copy the field Cam_Cmd 
	of data structure CamAck, the value ackcmd. CamAck is the data structure used by the 
	function Cam_SendCmd (),to prepare the transfer of the ACK command.
*/
unsigned char Cam_AckCmd(unsigned long int ackcmd)
{
     CAMCMD CamAck;

     CAMSTATUS = CAMSTATUS & 0xFFFE; // CAMSTATUS Initialization
     
	 CamAck.Id = CAM_CMD_ACK;
     CamAck.Cam_Cmd = ackcmd;
     
     return Cam_SendCmd(&CamAck);
}

/* 	unsigned char Cam_PackSizeCmd(void):
   	This function modifies the main global variable CAMSTATUS and reads the information contained 
	in the variable CAMFRAMESIZE, initially modified by the user, to set the field Cam_Cmd of data 
	structure CamPackSize. In particular, the function goes to read the information on the package 
	size used for the JPEG images transmission. CamPackSize is the structure used by the function 
	Cam_SendCmd () to prepare the transfer of the  SET SIZE PACKAGE command.
*/
unsigned char Cam_PackSizeCmd(void)
{
     unsigned long int lowbyte, highbyte, packsize;
     CAMCMD CamPackSize;
     
	 CAMSTATUS = CAMSTATUS & 0xFFFE; // CAMSTATUS Initialization

     packsize = CAMFRAMESIZE & PACKSIZE_NUM;
     
     if(packsize<16)
     {
         packsize = 16;
         CAMFRAMESIZE = CAMFRAMESIZE & 0xFC00;
         CAMFRAMESIZE = CAMFRAMESIZE | 0x0010; // packsize=16
     }
     else if(packsize>512)
     {
         packsize = 512;
         CAMFRAMESIZE = CAMFRAMESIZE & 0xFC00;
         CAMFRAMESIZE = CAMFRAMESIZE | 0x0200; // packsize=512
     }
     
     highbyte = (packsize>>8)&0x000000FF;
     lowbyte = packsize & 0x000000FF;
     
     CamPackSize.Id = CAM_CMD_SETPACKETSIZE;
     CamPackSize.Cam_Cmd = 0x08000000 +(lowbyte<<16) + (highbyte<<8);
     
     return Cam_SendCmd(&CamPackSize);
}

/* 	unsigned char Cam_GetCmd(void):
   	This function modifies the main global variable CAMSTATUS and reads the information contained 
	in the variable CAMCONFIG, initially modified by the user, to set the field Cam_Cmd of data 
	structure CamGet. In particular, the function can read the information about the requested 
	streaming type (video or snapshot). CamGet is the structure used by the function Cam_SendCmd (),
	to prepare the transfer of the GET PICTURE command.
*/
unsigned char Cam_GetCmd(void)
{
     unsigned long int pictype;
     CAMCMD CamGet;

	 CAMSTATUS = CAMSTATUS & 0xFFFE; // CAMSTATUS Initialization
     
     pictype = (CAMCONFIG>>PICTYPE_FLAG) & PICTYPE_NUM;
     if( pictype == DATATYPE_JPEG_PREVIEW ) pictype = 0x05;
     
     CamGet.Id = CAM_CMD_GETPICTURE;
     CamGet.Cam_Cmd = pictype<<24;
     
     return Cam_SendCmd(&CamGet);
}

/* 	unsigned char Cam_ResetCmd(unsigned long int restype,unsigned long int spreset):
   	This function modifies the main global variable CAMSTATUS and uses the arguments to set the 
	field Cam_Cmd of data structure CamReset to specify the desired reset command type. 
	CamReset is the structure used by the function Cam_SendCmd (),to prepare the transfer of the RESET command.
*/
unsigned char Cam_ResetCmd(unsigned long int restype,unsigned long int spreset)
{
     CAMCMD CamReset;
     CAMSTATUS = CAMSTATUS & 0xFFFE; // CAMSTATUS Initialization

     if( (restype!=RESET_SYSTEM && restype!=RESET_STATE_MACHINE) || 
         (spreset!=SIMPLE_RESET && spreset!=SPECIAL_RESET) ) return 0; 
     
     CamReset.Id = CAM_CMD_RESET;
     CamReset.Cam_Cmd = (restype<<24) + spreset;
     
     return Cam_SendCmd(&CamReset);
}

/* 	unsigned char Cam_PowerOffCmd(void):
   	This function modifies the main global variable CAMSTATUS and prepares the CamPowerOff command. 
	CamPowerOff once again represents the structure used by the function Cam_SendCmd (),to prepare the transfer 
	of the POWER OFF command. Please note that this command determines the passage in CAMERA SLEEP mode 
	causing inevitably loss of synchronization.
*/
unsigned char Cam_PowerOffCmd(void)
{
     CAMCMD CamPowerOff;
	 
     CamPowerOff.Id = CAM_CMD_POWEROFF;
     CamPowerOff.Cam_Cmd = 0x00000000;
     
     if(Cam_SendCmd(&CamPowerOff))
     { 
         CAMSTATUS = 0; // Camera Synchronization lost.
         return 1;
     } else return 0;
}

/* 	unsigned char Cam_SnapshotCmd(void):
   	This function modifies the main global variable CAMSTATUS and reads the information contained in the 
	variable CAMCONFIG and in the variable CAMDROPFRAMES, to set the Cam_Cmd field of data structure CamSnap. 
	In particular, the function can read the information about the compression type contained in the variable
	CAMCONFIG and the number of dropped frames contained in the variable CAMDROPFRAMES. CamSnap is the structure 
	used by the function Cam_SendCmd (),to prepare the transfer of the SNAPSHOT command.
*/
unsigned char Cam_SnapshotCmd(void)
{
     unsigned long int snaptype, snapskipLB, snapskipHB;
     CAMCMD CamSnap;

     CAMSTATUS = CAMSTATUS & 0xFFFE; // CAMSTATUS Initialization

     snaptype = (CAMCONFIG>>SNAPTYPE_FLAG) & SNAPTYPE_NUM;
     snapskipLB = (unsigned long int)(CAMDROPFRAMES & 0x00FF);
     snapskipHB = (unsigned long int)((CAMDROPFRAMES>>8) & 0x00FF);
     
     CamSnap.Id = CAM_CMD_SNAPSHOT;
     CamSnap.Cam_Cmd = (snaptype<<24) + (snapskipLB<<16) + (snapskipHB<<8);
     
     return Cam_SendCmd(&CamSnap);
}

/* 	Sync_Cmd - Read me:
   	This function modifies the main global variable CAMSTATUS and creates the CamSync command. 
	CamSync is the structure used by the function Cam_SendCmd (),to prepare the transfer 
	of the SYNCHRONIZATION command.
*/
unsigned char Cam_SyncCmd(void)
{
     CAMCMD CamSync;

	 CAMSTATUS = 0; // CAMSTATUS Initialization
     
	 CamSync.Id = CAM_CMD_SYNC;
     CamSync.Cam_Cmd = 0x00000000;
     
     return Cam_SendCmd(&CamSync);
}


/*****************************************************************************/
/* 	void UART1_Config(unsigned int baudvalue):
   	This function configures the dsPIC33F UART1 module. In particular it enables microcontroller to 
	receive the channel Rx ISR (Tx interrupt channel is unused) every time the buffer is full (4 bytes). 
	The UART1 baud-rate value must be passed as argument. Finally, the chosen serial communication type 
	has 8-bit data for the field, 1 stop bit and no parity bits.
*/
void UART1_Config(unsigned int baudvalue)
{
	unsigned int U1MODEvalue; // Holds the value of uart config reg 
	unsigned int U1STAvalue;  // Holds the information regarding uart TX & RX interrupt modes 
	CloseUART1(); // Turn off UART1 module 
	
	// Configure uart1 receive and transmit interrupt 
	IFS0bits.U1RXIF = 0;
	ConfigIntUART1(UART_RX_INT_EN & UART_RX_INT_PR3 & UART_TX_INT_DIS & UART_TX_INT_PR2);
	
	U1MODEvalue = 	UART_EN & UART_UEN_00 &  UART_IrDA_DISABLE & UART_BRGH_SIXTEEN & UART_UXRX_IDLE_ONE & UART_IDLE_CON & UART_DIS_WAKE
			  		& UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_NO_PAR_8BIT & UART_1STOPBIT;
	U1MODEvalue = U1MODEvalue & 0xFFF9; //becuase the UART_NO_PAR_8BIT mask gives a wrong result
	U1STAvalue = UART_TX_ENABLE  & UART_ADR_DETECT_DIS & UART_SYNC_BREAK_DISABLED & UART_INT_RX_BUF_FUL & UART_IrDA_POL_INV_ZERO;
	OpenUART1(U1MODEvalue, U1STAvalue, baudvalue);
	return;
}

/* 	void UART2_Config(unsigned int baudvalue):
   	This function configures the dsPIC33F UART2 module using also Microchip C30 functions. 
	In particular it enables microcontroller to receive the channel Rx ISR (Tx interrupt channel 
	is unused) every time a new byte is arrived. The UART2 baud-rate value must be passed as argument. 
	Finally, the chosen serial communication type has 8-bit data for the field, 1 stop bit and no parity bits.
*/
void UART2_Config(unsigned int baudvalue)/* Holds the value of baud register */
{
	   
	unsigned int U2MODEvalue; /* Holds the value of uart config reg */
	unsigned int U2STAvalue;  /* Holds the information regarding uart TX & RX 
                                 interrupt modes */

	CloseUART2(); /* Turn off UART2 module */
	
	/* Configure uart2 receive and transmit interrupt */
	IFS1bits.U2RXIF = 0;
	ConfigIntUART2(UART_RX_INT_EN & UART_RX_INT_PR5 & UART_TX_INT_DIS & UART_TX_INT_PR2);
	
	U2MODEvalue = 	UART_EN & UART_UEN_00 &  UART_IrDA_DISABLE & UART_BRGH_SIXTEEN & UART_UXRX_IDLE_ONE & UART_IDLE_CON & UART_DIS_WAKE
			  		& UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_NO_PAR_8BIT & UART_1STOPBIT;
	U2MODEvalue= U2MODEvalue & 0xFFF9; //becuase the UART_NO_PAR_8BIT mask gives a wrong result
	U2STAvalue = UART_TX_ENABLE  & UART_ADR_DETECT_DIS & UART_SYNC_BREAK_DISABLED & UART_INT_RX_CHAR & UART_IrDA_POL_INV_ZERO;
	OpenUART2(U2MODEvalue, U2STAvalue, baudvalue);
	return;
}

/* 	void Cam_TIMER89Config(unsigned long int match_value):
	This function configures the 32-bit dsPIC33F T89 timer using also Microchip C30 functions. 
	The period must be passed as argument and when the current value of the timer coincides with the period, 
	the module is configured to activate an ISR T9. The ISR, which can be found in the file CameraMain.c, 
	writes 0 in the WAIT flag to communicate that the timer has completed the period at least once.
*/
void Cam_TIMER89Config(unsigned long int match_value)
{
  	// Enable Timer89 Interrupt and Priority to "1" 
    // This function clears the 16-bit Interrupt Flag (TxIF) bit and then sets the
    // interrupt priority and enables/disables the interrupt.
  	IFS3bits.T9IF = 0; // Clear Timer9 interrupt flag  
  	Cam_ConfigIntTimer89(T9_INT_PRIOR_4 & T9_INT_ON);
  
  	// This function writes the 32-bit value into the Timer register. 
  	Cam_WriteTimer89(0);
  	// This function configures the 16-bit Timer Control register and sets the
    // period match value into the PR register. 
  	// match_value = 0xFFFFFF;   
  	Cam_OpenTimer89(T8_OFF & T8_GATE_OFF & T8_PS_1_256 & T8_32BIT_MODE_ON & T8_SOURCE_INT, match_value);
  	return;
}

/* void Cam_CloseTimer(void):
   This function closes timer9.
*/
void Cam_CloseTimer(void)
{
    T9CONbits.TON = 0;                      /* Disable timer9 */
    IFS3bits.T9IF = 0;                      /* Clear Timer9 interrupt flag */
}

/* void Send_by_UART2(unsigned int *buffer, unsigned char n):
   This function puts transfer data in the UART2-TX buffer.
   The UART module has 4-deep First-In-First-Out (FIFO) Transmit Data Buffer
   and the function can wait, if the buffer is full, to complete the 6-deep 
   command transmission.
*/
void Send_by_UART2(unsigned int *buffer, unsigned char n)
{
    char* temp_ptr = (char*) buffer;
    /* transmit till NULL character is encountered */
    if(U2MODEbits.PDSEL == 3)       	/* check if TX is 8bits or 9bits */
    {
        while(*buffer != '\0') 
        {
            while(U2STAbits.UTXBF); 	/* wait if the buffer is full */
            U2TXREG = *buffer++;    	/* transfer data word to TX reg */
        }
    }
    else
    {
        // while(*temp_ptr != '\0')
        int i;
        for(i=0; i<n; i++)
        {
            while(U2STAbits.UTXBF);  	/* wait if the buffer is full */
            U2TXREG = *temp_ptr++;   	/* transfer data byte to TX reg */
        }
    }
    return;
}

/* void Send_by_UART1(unsigned int *buffer, unsigned char n):
   This function puts transfer data in the UART1-TX buffer.
   The UART module has 4-deep First-In-First-Out (FIFO) Transmit Data Buffer
   and the function can wait, if the buffer is full, to complete the transmission.
*/
void Send_by_UART1(unsigned int *buffer, unsigned char n)
{
    char* temp_ptr = (char*) buffer;
    /* transmit till NULL character is encountered */
    if(U1MODEbits.PDSEL == 3)       	/* check if TX is 8bits or 9bits */
    {
        while(*buffer != '\0') 
        {
            while(U1STAbits.UTXBF); 	/* wait if the buffer is full */
            U1TXREG = *buffer++;    	/* transfer data word to TX reg */
        }
    }
    else
    {
        // while(*temp_ptr != '\0')
        int i;
        for(i=0; i<n; i++)
        {
            while(U1STAbits.UTXBF);  	/* wait if the buffer is full */
            U1TXREG = *temp_ptr++;   	/* transfer data byte to TX reg */
        }
    }
    return;
}

void Cam_CloseTimer89(void)
{
    IEC3bits.T9IE = 0;      /* Disable the Timer9 interrupt */
    if (T8CONbits.T32 == 1)
        T8CONbits.TON = 0;  /* Disable Timer 8 */
    IFS3bits.T9IF = 0;      /* Clear Timer9 interrupt flag */
	return;
}

void Cam_WriteTimer89( unsigned long int tval)
{
    TMR9HLD = (tval>>16);
    TMR8 = tval;       /* LSB to Timer8 and MSB to Timer9HLD */    
    return; 
}

void Cam_OpenTimer89(unsigned int tconfig,unsigned long int tper)
{
    TMR8 = 0;               /* Reset Timer8 to 0x0000 */
    TMR9 = 0;               /* Reset Timer9 to 0x0000 */
	//TMR9HLD = 0;
    PR8 = tper;           /* assigning Period to PR8 */
    PR9 = tper>>16;       /* Period to  PR9Register */
    T8CON =  tconfig;        /* Configure timer control reg */
    T8CONbits.T32 = 1;
	return;
}

void Cam_ConfigIntTimer89(unsigned int tconf)
{
    IFS3bits.T9IF = 0;                   /* clear IF bit */
    IPC13bits.T9IP = (tconf &0x0007);    /* assigning Interrupt Priority */
    IEC3bits.T9IE = (tconf &0x0008)>>3;  /* Interrupt Enable /Disable */
	return;
}
/*****************************************************************************/
/* 	unsigned char Cam_ReadData(void):
   	This function starts every time a new byte, sent by the camera, is inserted into the buffer. 
	This function starts inside the UART2 Rx interrupt routine.
	The function execution depends on the IMAGE flag because, according to the microcontroller is awaiting 
	a command or an image, it must to follow different paths. If the microcontroller is awaiting a command 
	the Cam_Readmd() function starts after the sixth received byte to recognize the command type, 
	while if the microcontroller is awaiting an image the Cam_ReadDataImage() function starts after receipt 
	of each image byte to store the received byte in the global vector FRAME. Another important feature of 
	this function is the proper and fast setting of the IMAGE flag to avoid losing consistency as soon as the 
	camera starts to send image data.

*/
unsigned char Cam_ReadData(void)
{        
    unsigned char xret=1;

	//rst[0] = (unsigned int)(ReadUART2()&0x00FF);
    CAMBUF[CAMRBCOUNTER] = (unsigned int)(ReadUART2()&0x00FF);
	CAMRBCOUNTER++;
    
	if( (CAMSTATUS>>IMAGE_FLAG)&IMAGE_NUM )
    {
        CAMRBCOUNTER = 0;
        xret = Cam_ReadDataImage();
    }
    else if( CAMRBCOUNTER == 6 )
    {
		if( (CAMSTATUS>>DATA_FLAG)&DATA_NUM ) 
		    CAMSTATUS = CAMSTATUS | (IMAGE_NUM<<IMAGE_FLAG);     /* Set image flag */
		CAMRBCOUNTER = 0;
        xret = Cam_ReadCmd();
    }
    return xret;
}

/* unsigned char Cam_ReadCmd(void):
   This function can recognize the received command type (sent by the camera).
*/
unsigned char Cam_ReadCmd(void) 
{ 
    unsigned int Cmd_ID;

    Cmd_ID = ((unsigned int)(CAMBUF[0]))<<8;
    Cmd_ID = Cmd_ID + (unsigned int)(CAMBUF[1]);
	
    CAMSTATUS = CAMSTATUS | (READY_NUM<<READY_FLAG);      /* Set Ready command flag */
  
    if(Cmd_ID == CAM_CMD_ACK)
    {                                        
        CAMSTATUS = CAMSTATUS & (~(ACK_NUM<<ACK_FLAG));     /* Clear ACK command flag */ 
        return 1;
    }
    if(Cmd_ID == CAM_CMD_NAK)
    {
        CAMSTATUS = CAMSTATUS & (~(NAK_NUM<<NAK_FLAG));     /* Clear NAK command flag */
        CAMSTATUS = CAMSTATUS & (~(ERROR_NUM<<ERROR_FLAG));     /* Clear err command flag */
		CAMSTATUS = CAMSTATUS + (CAMBUF[4]<<ERROR_FLAG);  // Error number
        return 0;
    }
    if(Cmd_ID == CAM_CMD_DATA)
    {
        CAMSTATUS = CAMSTATUS & (~(DATA_NUM<<DATA_FLAG));     /* Clear DATA command flag */
        return 1;
    }
    if(Cmd_ID == CAM_CMD_SYNC)
    {
        CAMSTATUS = CAMSTATUS & (~(SYNC_NUM<<SYNC_FLAG));     /* Clear SYNC command flag */
        return 1;
    }
    if( (CAMSTATUS>>ERROR_FLAG)&ERROR_NUM ) return 0;
  
    Cam_SWError(CAM_ERR_SW_ERROR0);
    return 0;
}

/* unsigned char Cam_ReadDataImage(void):
   This function is designed to store image bytes in the FRAME vector resetting simultaneously the T89 timer.
*/
unsigned char Cam_ReadDataImage(void)
{
    unsigned char check=0; 
    unsigned int packsize, pictype, snaptype;
//	unsigned char st[4];
    
	T8CONbits.TON = 0;      /* Disable Timer 8 */
	Cam_WriteTimer89(0);

	packsize = CAMFRAMESIZE & PACKSIZE_NUM;
    pictype = (CAMCONFIG >> PICTYPE_FLAG) & PICTYPE_NUM;  
    snaptype = (CAMCONFIG>> SNAPTYPE_FLAG) & SNAPTYPE_NUM;
	
	T8CONbits.TON = 1;      /* Turn on Timer89 */
    
    if(CTYPE) FRAME[CAMCOUNTER] = CAMBUF[0];
	else FRAME[BCOUNTER] = CAMBUF[0];
    CAMCOUNTER++;
    BCOUNTER++;
    
	check= ( ((unsigned long int)BCOUNTER)==IMAGESIZE );
    if( (CAMCOUNTER >= packsize) || (check && BCOUNTER!=0) ) 
    {
		if( (pictype==DATATYPE_PREVIEW) || (pictype==DATATYPE_SNAPSHOT && snaptype==UNCOMPRESSED_PICTURE) )
        {
			if(check)
			{
				//CAMCOUNTER = 0; 
				CAMSTOP=1; 
				//CAMSTATUS = CAMSTATUS & (~(IMAGE_NUM<<IMAGE_FLAG));     /* Clear IMAGE flag */ 
			}
			CAMCOUNTER = 0;
			SEND=1;
		}
        else if( CAMCOUNTER>packsize )
        {
            Cam_SWError(CAM_ERR_SW_ERROR2);
         	return 0;
        }
        else
		{ 
			//st[0]=0xAA;
			//st[1]=(((unsigned int)JPEGFRAME)>>8) & 0x00FF;
			//st[2]=((unsigned int)JPEGFRAME) & 0x00FF;
			//Send_by_UART1((unsigned int *)st,3);
			CAMSTATUS = CAMSTATUS | (READY_NUM<<READY_FLAG);      /* Set Ready command flag */	
			BCOUNTER = BCOUNTER-6;
			JPEGDIM=CAMCOUNTER;
			CAMCOUNTER = 0;
			if(check)
			{ 
				CAMSTOP=1; 
				CAMSTATUS = CAMSTATUS & (~(IMAGE_NUM<<IMAGE_FLAG));     /* Clear IMAGE flag */ 
			}//else...
		}
    }
    return 1;
}
/******************************************************************************/
/* unsigned char Cam_SendCmd(CAMCMD *Hex_Cmd):
   This function creates a string of 6 bytes to send the commands to the camera by the function Send_by_UART2() .             
*/
unsigned char Cam_SendCmd(CAMCMD *Hex_Cmd) 
{    
    char sc[6];
    while(BusyUART2());
	//while(BusyUART1());
    if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
    else
    { 
        sc[0] = (char)( ((Hex_Cmd->Id)>>8) & 0x00FF );
        sc[1] = (char)(  (Hex_Cmd->Id) & 0x00FF );
        sc[2] = (char)( ((Hex_Cmd->Cam_Cmd)>>24) & 0x00FF );
        sc[3] = (char)( ((Hex_Cmd->Cam_Cmd)>>16) & 0x00FF ); 
        sc[4] = (char)( ((Hex_Cmd->Cam_Cmd)>>8) & 0x00FF ); 
        sc[5] = (char)(  (Hex_Cmd->Cam_Cmd) & 0x00FF ); 
		
		Cam_Sleep(0x00000098); // 1ms
		//Send_by_UART1((unsigned int *)sc,6); // Characters transimitted in hex.
       	Send_by_UART2((unsigned int *)sc,6); // Characters transimitted in hex.	
       	//while(BusyUART2()); 
    }
    return 1;
}

/* 	unsigned char Cam_SendPicture(void):
   	This function sends uncompressed image packages to a remote PC although the image transfer 
	from the camera is not yet finished (to avoid loss of synchronization between two image requests).
*/
unsigned char Cam_SendPicture(void)
{
    //unsigned int i=0, 
	unsigned int packsize;
    char sd[6];

	if(SEND==1)
	{
    	SEND=0;
		packsize = CAMFRAMESIZE & PACKSIZE_NUM;
		NEWLIMIT+=(unsigned long int)packsize;
		if(NEWLIMIT>IMAGESIZE) NEWLIMIT=IMAGESIZE;
		//sd[0] = (NEWLIMIT>>16) & 0x00FF;
		//sd[1] = (NEWLIMIT>>8) & 0x00FF;
		//sd[2] = NEWLIMIT & 0x00FF;
		//Send_by_UART1((unsigned int *)sd,3);
	}
	while(BusyUART1());
	if(TXCOUNTER<NEWLIMIT)
	{
    	sd[0] = FRAME[TXCOUNTER];
    	Send_by_UART1((unsigned int *)sd,1);
		TXCOUNTER++;
		if(TXCOUNTER==IMAGESIZE && IMAGESIZE!=0) return 0;
	} 
    return 1;
}

/* unsigned char Cam_SendJPEGFrame(unsigned char send, unsigned int id):
   This function check if the JPEG image packet is correct, with Cam_ReadJPEGFrame(), and then sends out the packet. 
*/
unsigned char Cam_SendJPEGFrame(unsigned char send, unsigned int id)
{
    unsigned char xret;//,i;
	unsigned char st[4];
//	unsigned int packsize;
    unsigned int LOCALCOUNTER;
	//st[0]=0xCC;
	//st[1]=(((unsigned int)JPEGFRAME)>>8) & 0x00FF;
	//st[2]=((unsigned int)JPEGFRAME) & 0x00FF;
	//Send_by_UART1((unsigned int *)st,3);
	
	LOCALCOUNTER = 4;
	xret = Cam_ReadJPEGFrame(id);
	if(!xret)	return 0;
    
    while(LOCALCOUNTER < JPEGDIM-2)
    {
        if( IMAGECOUNTER==(IMAGESIZE-1) ) //if( IMAGECOUNTER==(IMAGESIZE-1) )
        {
            if( LOCALCOUNTER==(JPEGDIM-3) ) //(FRAMETX.dim-3) )
                CAMSTOP=1;
            else
            {
                Cam_SWError(CAM_ERR_SW_ERROR4);
         		return 0;
            }
        }
        st[0] = FRAME[LOCALCOUNTER++];
        Send_by_UART1((unsigned int *)st,1);
		IMAGECOUNTER++; 
    } 
      
    return 1;
}

/**********************************************************************************************************/
/* void Cam_Init(void):
   This feature assigns default values to the main global variables to perform a new image stream. 
*/
void Cam_Init(void)
{
    JPEGDIM = 0;
	CAMSTATUS = CAMSTATUS & 0x0001;  
    CAMCOUNTER = 0;                        
    CAMRBCOUNTER = 0;              
    IMAGECOUNTER = 0;  
    BCOUNTER = 0;   
	TXCOUNTER=0;
	SEND=0;          
    IMAGESIZE = 0;                    
    CAMSTOP = 0;    
	NEWLIMIT = 0;             
    return;
}

/* void Cam_Turnoff(void):
   This feature closes the application disabling the UART2 and T89 peripherals.
*/
void Cam_Turnoff(void)
{
    CloseUART2();
    Cam_CloseTimer89();
    return;
}

/* 	unsigned char Cam_GetStream(void) :
	This function initially calls the function Cam_Init () described above and then, 
	reads the CAMCONFIG bits to activate the correct sequence according to the requests. 
	
*/
unsigned char Cam_GetStream(void)     
{
         unsigned char xret = 0; 
         unsigned int pictype, snaptype;
         
         Cam_Init();
         
         pictype = (CAMCONFIG>>PICTYPE_FLAG) & PICTYPE_NUM;
         if( pictype==DATATYPE_SNAPSHOT )
         {
             snaptype = (CAMCONFIG>>SNAPTYPE_FLAG) & SNAPTYPE_NUM;
             xret = Cam_GetPicture(snaptype);
         }
         else if(pictype==DATATYPE_PREVIEW)
                  xret = Cam_GetVideo(UNCOMPRESSED_VIDEO);
              else if(pictype==DATATYPE_JPEG_PREVIEW)
                       xret = Cam_GetVideo(COMPRESSED_VIDEO);
                    else    return 0;
    	 
		 //Cam_SendInfo();	
		 Cam_PowerOffCmd();
		 if(xret) CAMSTATUS=0;
         return xret;
}

/* 	unsigned char Cam_GetPicture(unsigned char type):  
   	This function calls the function Cam_UncompressePicture () or the function Cam_JpegPicture () 
	in according to the value of the argument.
*/
unsigned char Cam_GetPicture(unsigned char type)    
{
         if( type == UNCOMPRESSED_PICTURE )
             return Cam_UncompressedPicture();
         else if(type == COMPRESSED_PICTURE)
                  return Cam_JpegPicture();
              else return 0;
}

/* 	unsigned char Cam_GetVideo(unsigned char type):
   	This function calls the function Cam_UncompresseVideo () or the function Cam_JpegVideo () 
	in according to the value of the argument. 
*/
unsigned char Cam_GetVideo(unsigned char type)     
{
         if( type == COMPRESSED_VIDEO )
             return Cam_JpegVideo();
         else if( type == UNCOMPRESSED_VIDEO )
                  return Cam_UncompressedVideo();
              else return 0;
}

/* 	unsigned char Cam_JpegPicture(void):
   	This function implements the JPEG image acquisition procedure as described in the figure reported 
	at the end of paragraph 2.     
*/
unsigned char Cam_JpegPicture(void)
{
         unsigned char xret, send;
         unsigned int ind=0;

		 BCOUNTER=-6;
         CTYPE=1;

         send = (CAMCONFIG>>SEND_FLAG)&SEND_NUM;
         
		 while(ind<5)
		 {
			 ind++;
             xret = Cam_InitialCmd();                  //(AA 01 00 07 07 07)
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(ACK_FLAG, CAM_CMD_INITIAL, 0x0000FFFF);  //(AA 0E 01 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;

		 
		 while(ind<5)
		 {
			 ind++;
             xret = Cam_PackSizeCmd();                 //(AA 06 08 00 02 00)
         	 if(!xret) continue;//return 0;
         
         	 xret =  Cam_Wait(ACK_FLAG, CAM_CMD_SETPACKETSIZE, 0x0000FFFF);  //(AA 0E 06 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;

                     
	     while(ind<5)
		 {
			 ind++;
             xret = Cam_SnapshotCmd();                 //(AA 05 00 00 00 00)
         	 if(!xret) continue;//return 0;
         
         	 xret =  Cam_Wait(ACK_FLAG, CAM_CMD_SNAPSHOT, 0x0000FFFF);  //(AA 0E 05 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;   
         
		 while(ind<5)
		 {
			 ind++;
             xret = Cam_GetCmd();                      //(AA 04 01 00 00 00)
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(ACK_FLAG, CAM_CMD_GETPICTURE, 0x0000FFFF);  //(AA 0E 04 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(DATA_FLAG, DATATYPE_SNAPSHOT, 0x0000FFFF);  //(AA 0A 01 ~~ ~~ ~~) data
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
			 else break;
		 }
		 if(!xret) return 0;
         ind=0;
         
         while(1)
         {
             xret = Cam_AckCmd( (ind<<8)+(ind>>8) );                     //(AA 0E 00 00 00 00)
         	 if(!xret) continue;//return 0;
			
			 xret = Cam_Wait(IMAGE_FLAG, ind, 0x0000FFFF);  // Image packet
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
             if(!xret) continue;//return 0;
  
             xret = Cam_SendJPEGFrame(send,ind);
             if(!xret) return 0;
             
             if(CAMSTOP) break;
                 
             ind++;
             //xret = Cam_AckCmd( (ind<<8)+(ind>>8) );          
													//(AA 0E 00 00 01 00) 
                                                    //(AA 0E 00 00 02 00)
                                                    //...
                                                    //(AA 0E 00 00 F0 F0)
             //if(!xret) return 0;               
         }
		 while(BusyUART2());
         if(xret) xret = Cam_AckCmd(0x0000F0F0); // Final ACK
		 while(BusyUART2());
		 if(xret) xret = Cam_AckCmd(0x0000F0F0); // Final ACK
		 //xret = Cam_ResetCmd(RESET_SYSTEM,SIMPLE_RESET);		
		 return xret;        
}

/* 	unsigned char Cam_UncompressedPicture(void):
   	This function implements the uncompressed image acquisition procedure as described in the figure 
	reported at the end of paragraph 2.    
*/
unsigned char Cam_UncompressedPicture(void)
{
         unsigned char xret, send;
		 unsigned char ind=0;
         
         send = (CAMCONFIG>>SEND_FLAG)&SEND_NUM;
		 CTYPE=0;

		while(ind<5)
		 {
			 ind++;
             xret = Cam_InitialCmd();                  //(AA 01 00 06 07 07)  
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(ACK_FLAG, CAM_CMD_INITIAL, 0x0000FFFF);  //(AA 0E 01 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;
                     

		while(ind<5)
		 {
			 ind++;
             xret = Cam_SnapshotCmd();                 //(AA 05 01 00 00 00)
         	 if(!xret) continue;//return 0;
         
         	 xret =  Cam_Wait(ACK_FLAG, CAM_CMD_SNAPSHOT, 0x0000FFFF);  //(AA 0E 05 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;   

		 while(ind<5)
		 {
			 ind++;
             xret = Cam_GetCmd();                      //(AA 04 01 00 00 00)
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(ACK_FLAG, CAM_CMD_GETPICTURE, 0x0000FFFF);  //(AA 0E 04 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(DATA_FLAG, DATATYPE_SNAPSHOT, 0x0000FFFF);  //(AA 0A 01 ~~ ~~ ~~) data
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
			 else break;
		 }
	
         xret = Cam_Wait(IMAGE_FLAG, 0, 0x000FFFFF);  // Image packet
         if(!xret) return 0;
 		 
		 while(BusyUART2());
		 xret = Cam_AckCmd(0x0A000000);                     //(AA 0E 0A xx 00 00)   
		 while(BusyUART2());
		 xret = Cam_AckCmd(0x0A000000);                     //(AA 0E 0A xx 00 00)
		                  
         //if(!xret) return 0;
	
         //xret = Cam_SendPicture();
         //if(!xret) return 0;
         
		 //xret = Cam_ResetCmd(RESET_SYSTEM,SIMPLE_RESET);
         return xret;     
}


/* 	unsigned char Cam_JpegVideo(void):
   	This function implements the JPEG video acquisition procedure as described in the figure 
	reported at the end of paragraph 2. 
*/
unsigned char Cam_JpegVideo(void)
{
         unsigned char xret, ind=0, k=0, send=1;
		 send = (CAMCONFIG>>SEND_FLAG)&SEND_NUM;
		 CTYPE=1;
         
		 while(ind<5)
		 {
			 ind++;
             xret = Cam_InitialCmd();                  //(AA 01 00 07 07 07)
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(ACK_FLAG, CAM_CMD_INITIAL, 0x0000FFFF);  //(AA 0E 01 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;

		 while(ind<5)
		 {
			 ind++;
             xret = Cam_PackSizeCmd();                 //(AA 06 08 00 02 00)
         	 if(!xret) continue;//return 0;
         
         	 xret =  Cam_Wait(ACK_FLAG, CAM_CMD_SETPACKETSIZE, 0x0000FFFF);  //(AA 0E 06 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;

         start1=1;

         while(k<VIDEOSIZE)
         {
			 while(!start1);
			 start1=0;
			 BCOUNTER=-6;
			 ind=0;
			 
			 while(ind<5)
		 	 {
			 	ind++;
             	xret = Cam_GetCmd();                      //(AA 04 01 00 00 00)
         	 	if(!xret) continue;//return 0;
         
         	 	xret = Cam_Wait(ACK_FLAG, CAM_CMD_GETPICTURE, 0x0000FFFF);  //(AA 0E 04 xx 00 00) ack
			 	if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 	if(!xret) continue;//return 0;
         
         	 	xret = Cam_Wait(DATA_FLAG, DATATYPE_JPEG_VIDEO, 0x0000FFFF);  //(AA 0A 01 ~~ ~~ ~~) data
			 	if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 	if(!xret) continue;//return 0;
			 	else break;
		 	}
		 	if(!xret) return 0;
         	ind=0;

//			 xret = Cam_GetCmd();                       //(AA 04 05 00 00 00)
//             if(!xret) return 0;
//         
//             xret = Cam_Wait(ACK_FLAG, CAM_CMD_GETPICTURE, 0x0000FFFF);   //(AA 0E 04 xx 00 00) ack
//             if(!xret) return 0;
//         
//             xret = Cam_Wait(DATA_FLAG, DATATYPE_JPEG_VIDEO, 0x0000FFFF);   //(AA 0A 05 ~~ ~~ ~~) data
//             if(!xret) return 0;
             
            while(BusyUART1());
             
			while(1)
         	{
             	xret = Cam_AckCmd( (ind<<8)+(ind>>8) );                     //(AA 0E 00 00 00 00)
         	 	if(!xret) continue;//return 0;
			
			 	xret = Cam_Wait(IMAGE_FLAG, ind, 0x0000FFFF);  // Image packet
			 	if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
             	if(!xret) continue;//return 0;
  
             	xret = Cam_SendJPEGFrame(send,ind);
             	if(!xret) return 0;
             
             	if(CAMSTOP) break;
                 
             	ind++;
             	//xret = Cam_AckCmd( (ind<<8)+(ind>>8) );          
													//(AA 0E 00 00 01 00) 
                                                    //(AA 0E 00 00 02 00)
                                                    //...
                                                    //(AA 0E 00 00 F0 F0)
             	//if(!xret) return 0;               
         	}
		 	while(BusyUART2());
         	if(xret) xret = Cam_AckCmd(0x0000F0F0); // Final ACK
		 	while(BusyUART2());
		 	if(xret) xret = Cam_AckCmd(0x0000F0F0); // Final ACK
			k++;
			Cam_Init();
//             xret = Cam_AckCmd(ind);                       //(AA 0E 00 00 00 00)
//             if(!xret) return 0;
//         
//             while(1)
//             {
//                 xret = Cam_Wait(IMAGE_FLAG, ind, 0x0000FFFF);  // Image packet
//                 if(!xret) return 0;
//  
//                 xret = Cam_SendJPEGFrame(1,ind);
//                 if(!xret) return 0;
//                 
//                 if(CAMSTOP) break;
//                 
//                 ind++;
//                 while(BusyUART1());
//                 xret = Cam_AckCmd( (ind<<8)+(ind>>8) );      //(AA 0E 00 00 01 00) 
//                                                    //(AA 0E 00 00 02 00)
//                                                    //...
//                                                    //(AA 0E 00 00 F0 F0)
//                 if(!xret) return 0;    
//             }
//             xret = Cam_AckCmd(0x0000F0F0); // Final ACK
//             if(!xret) break;
//             k++;
             //if(!Cam_HeapManager()) return 0;
//			 Cam_Init();
         }
         return xret;        
}


/* 	unsigned char Cam_UncompressedVideo(void):
   	This function implements the uncompressed video acquisition procedure as described in the figure 
	reported at the end of paragraph 2.
*/
unsigned char Cam_UncompressedVideo(void)
{
         unsigned char xret, ind=0, k=0;//, ind=0;
		 CTYPE=0;

		 while(ind<5)
		 {
			 ind++;
             xret = Cam_InitialCmd();                 //(AA 01 00 06 07 07)  
         	 if(!xret) continue;//return 0;
         
         	 xret = Cam_Wait(ACK_FLAG, CAM_CMD_INITIAL, 0x0000FFFF);  //(AA 0E 01 xx 00 00) ack
			 if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 if(!xret) continue;//return 0;
         	 else break;
		 }
		 if(!xret) return 0;
         ind=0;

         start1=1;
         
         while(k<VIDEOSIZE)// && k<100)
         {    
			 while(!start1);
			 start1=0;

//             xret = Cam_GetCmd();                          //(AA 04 02 00 00 00)
//             if(!xret) break;
//
			while(ind<5)
		 	{	
			 	ind++;
             	xret = Cam_GetCmd();                      //(AA 04 01 00 00 00)
         	 	if(!xret) continue;//return 0;
         
         	 	xret = Cam_Wait(ACK_FLAG, CAM_CMD_GETPICTURE, 0x0000FFFF);  //(AA 0E 04 xx 00 00) ack
			 	if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 	if(!xret) continue;//return 0;
         
         	 	xret = Cam_Wait(DATA_FLAG, DATATYPE_PREVIEW, 0x0000FFFF);  //(AA 0A 01 ~~ ~~ ~~) data
			 	if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) ) return 0;
         	 	if(!xret) continue;//return 0;
			 	else break;
		 	}
			ind=0;
         
//             xret = Cam_Wait(ACK_FLAG, CAM_CMD_GETPICTURE, 0x0000FFFF);      //(AA 0E 04 xx 00 00) ack
//             if(!xret) break; //continue;
//         
//             xret = Cam_Wait(DATA_FLAG, DATATYPE_PREVIEW, 0x0000FFFF);      //(AA 0A 02 ~~ ~~ ~~) data
//             if(!xret) break;
         
			 xret = Cam_Wait(IMAGE_FLAG, 0, 0x000FFFFF);  // Image packet
         	 if(!xret) break;
 
		 	 //xret = Cam_AckCmd(0x0A000000);                     //(AA 0E 0A xx 00 00)                    
         	 //if(!xret) break;
			 while(BusyUART2());
		 	 xret = Cam_AckCmd(0x0A000000);                     //(AA 0E 0A xx 00 00)   
		 	 while(BusyUART2());
		 	 xret = Cam_AckCmd(0x0A000000);                     //(AA 0E 0A xx 00 00)
         	 //xret = Cam_SendPicture();
         	 //if(!xret) break;
             //while(BusyUART1());
           
			 Cam_Init();
			 k++;
         }
         return xret;     
}
/**********************************************************************************************************/
/* 	unsigned char Cam_Synchronization(float Fcy)   :
   	This function makes connection with the camera performing the synchronization procedure. 
	Moreover, the function configures (making use of function Cam_RoundFloat ()) the UART2 module, 
	hich manages the serial bus for the communication with the camera, and the T89 timer to implement 
	the timed wait functions.
*/
unsigned char Cam_Synchronization(float Fcy)    
{
     unsigned char xret = 1, i=0, camerr;
	 unsigned int bdiv; 
     float baudvalue; 
     
	 Cam_Init();
     CAMFCY = Fcy;
	 CAMSTATUS = CAMSTATUS & (~(SYNCHRO_NUM<<SYNCHRO_FLAG));     /* Clear Syncronization flag */
     
     bdiv = (CAMCONFIG>>BAUDDIV_FLAG) & BAUDDIV_NUM;
     switch(bdiv)
     {
         case 0:
                bdiv = (unsigned int)((BR7200+1)*8);   // 7200 bps
                break;
         case 1:
                bdiv = (unsigned int)((BR9600+1)*8);   // 9600 bps (default)
                break;
         case 2:
                bdiv = (unsigned int)((BR14400+1)*8);  // 14400 bps 
                break;
         case 3:
                bdiv = (unsigned int)((BR19200+1)*8);  // 19200 bps
                break;
         case 4:
                bdiv = (unsigned int)((BR28800+1)*8);  // 28800 bps
                break;
         case 5:
                bdiv = (unsigned int)((BR38400+1)*8);  // 38400 bps
                break;
         case 6:
                bdiv = (unsigned int)((BR57600+1)*8);  // 57600 bps
                break;
         case 7:
                bdiv = (unsigned int)((BR115200+1)*8); // 115200 bps
                break;
         default:
                return 0;
     }
     
     baudvalue = 14745600.0/((float)bdiv);
     baudvalue = Fcy/16.0/baudvalue-1.0;
     baudvalue = Cam_RoundFloat(baudvalue);
     
     UART2_Config((unsigned int)baudvalue);
	 //UART2_Config(20);
	 Cam_CloseTimer89();
	 Cam_TIMER89Config(0);
	 
     while(1)
     { 
         i++;
		 camerr = (CAMSTATUS>>ERROR_FLAG)&ERROR_NUM;
         if( (i>100) || camerr )
         {
    	     xret=0; 
    	     if(camerr==CAM_ERR_COMMAND_HEADER_ERROR)
	    	     xret=1;
			 break;
		 }

         Cam_Sleep(0x00000FFF);

         xret = Cam_SyncCmd();
         if( !xret ) continue;
         
         xret = Cam_Wait(ACK_FLAG, CAM_CMD_SYNC, 0x0000FFFF);  // (AA 0E 0D xx 00 00) ACK 
		 if( !xret ) continue;
         
         xret = Cam_Wait(SYNC_FLAG, 0, 0x0000FFFF);  // (AA 0D 00 00 00 00) SYNC
         if( !xret ) break;
         
         xret = Cam_AckCmd(0x0D000000);// send final ACK command
         if( !xret ) continue;
         
         break; // Syncronization was successfully...
     }	
     if(xret) CAMSTATUS = 0x0001;
     else CAMSTATUS = CAMSTATUS & 0xFFFE;
     return xret;
}


/* Cam_RoundFloat - Read me:
   This function can round a float number.
*/
float Cam_RoundFloat(float x2rx)
{
      float fx, cx;
      fx = floorf(x2rx);
      cx = ceilf(x2rx);
      if( (x2rx-fx)<=(cx-x2rx) ) return fx;
      else return cx;
}
/*************************************************************************************************************/
/* 	unsigned char Cam_Wait(unsigned int numf, unsigned int id,unsigned long int stbTime):
   	This is certainly the most important function of the firmware. With this function a cyclic timed 
	wait function is realized.  This function is called every time the microcontroller is awaiting 
	for a command or an image. The function initially configures the T89. The first argument numf 
	specifies the expected command type and the second argument id is intended to specify any additional 
	information useful in the function Cam_Check (numf, id), for example the identifier of the command just 
	sent to the camera. The implementation of the cycle controls the occurrence of a certain event. 
	Possible escapes from this cycle are: 
	 there is an error, ERROR non-zero bytes; 
	 the T89 triggers an interrupt and resets the WAIT flag, so the maximum waiting time has expired and CAMSTOP is zero (image transmission is not yet finished); 
	 the received command is different than expected; 
	 expected command is arrived; 
*/                  
unsigned char Cam_Wait(unsigned int numf, unsigned int id,unsigned long int stbTime)
{     
     unsigned char xret = 0, waitf;
	// unsigned char st[4];

     T8CONbits.TON = 0;       /* Disable Timer 8 */
	 Cam_WriteTimer89(0);
	 PR8 = stbTime;           /* assigning Period to PR8 */
     PR9 = stbTime>>16;       /* Period to PR9Register */
     T8CONbits.T32 = 1;
      
	 CAMSTATUS = CAMSTATUS | (WAIT_NUM<<WAIT_FLAG);      /* Set timer flag */
	 CAMSTATUS = CAMSTATUS & (~(READY_NUM<<READY_FLAG));     /* Clear Ready command flag */

     CAMSTATUS = CAMSTATUS | (0x01<<numf);     /* Set response command flag */
     //T9CONbits.TON= 1;                         /* Turn on Timer9 */ 
	 T8CONbits.TON= 1;                         /* Turn on Timer89 */
	 
     while( ((CAMSTATUS>>numf)&0x01) && (((CAMSTATUS>>WAIT_FLAG)&WAIT_NUM) || CAMSTOP) && (((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM)==0) && (((CAMSTATUS>>READY_FLAG)&READY_NUM)==0) && Cam_SendPicture() );
  
	 //Cam_CloseTimer89();
	 T8CONbits.TON = 0;  /* Disable Timer 8 */
	 
	 waitf = ((CAMSTATUS>>WAIT_FLAG)&WAIT_NUM)==0;    
   	 CAMSTATUS = CAMSTATUS & (~(READY_NUM<<READY_FLAG));     /* Clear Ready command flag */

     if( ((CAMSTATUS>>ERROR_FLAG)&ERROR_NUM) )
     {
		 CAMSTATUS = CAMSTATUS & (~(WAIT_NUM<<WAIT_FLAG));      /* Clear timer flag */
		 CAMSTATUS = CAMSTATUS & (~(0x01<<numf));     			/* Clear exp command flag */
         return 0;
     }
     if( waitf && CAMSTOP==0 )
     { 
		 //CAMSTATUS = CAMSTATUS & (~(WAIT_NUM<<WAIT_FLAG));      /* Clear timer flag */
		 //CAMSTATUS = CAMSTATUS & (~(0x01<<numf));     			/* Clear exp command flag */
         CAMSTATUS=0x0001;
		 //if(id != CAM_CMD_SYNC && id!=CAM_CMD_GETPICTURE && numf!=DATA_FLAG && (numf!=IMAGE_FLAG || CAMCOUNTER!=0) ) Cam_SWError(CAM_ERR_SW_ERROR5);
         return 0;
     }
     if( numf==IMAGE_FLAG )
     {   
		 CAMSTATUS = 0x0001;
		 if(!CAMSTOP) CAMSTATUS = CAMSTATUS | (IMAGE_NUM<<IMAGE_FLAG);     /* Set image flag */ 
         return 1;   
     }
     if( ((CAMSTATUS>>numf) & 0x0001)==0 )
     {     
         CAMSTATUS = CAMSTATUS & (~(ERROR_NUM<<ERROR_FLAG)); 	/* Clear ERROR flags */
		 CAMSTATUS = CAMSTATUS & (~(WAIT_NUM<<WAIT_FLAG));     	/* Clear timer flag */ 
         xret = Cam_Check(numf,id);
         return xret;
     }
     else 
     {         
		 CAMSTATUS = CAMSTATUS & (~(WAIT_NUM<<WAIT_FLAG));      /* Clear timer flag */
		 CAMSTATUS = CAMSTATUS & (~(0x01<<numf));     			/* Clear exp command flag */
		 Cam_SWError(CAM_ERR_SW_ERROR7);
         return 0;   
     }     
     return 0;
}     
 
 
/* 	unsigned char Cam_Check(unsigned int numf, unsigned int id):
   	This function checks the correctness of the received ACK command and also stores the 
	image size value sent by the camera. 
*/  
unsigned char Cam_Check(unsigned int numf, unsigned int id)
{         
     //unsigned int packsize, pictype;
//	 unsigned int pictype;
     unsigned long int a,b,c;
//	 unsigned char st[3];

     if(numf==ACK_FLAG)
     {
		 if( CAMBUF[2] != (id&0x00FF) )
         {
              Cam_SWError(CAM_ERR_SW_ERROR8);
         	  return 0;
         }
     }
     if(numf==DATA_FLAG)
     {
        if( CAMBUF[2] != (id&0x00FF) ) 
        {
            Cam_SWError(CAM_ERR_SW_ERROR9);
         	return 0;
        }

        //packsize = CAMFRAMESIZE & PACKSIZE_NUM;
        //pictype = (CAMCONFIG>>PICTYPE_FLAG) & PICTYPE_NUM;  
        //FRAMERX.p = (unsigned char *)calloc(packsize+10, sizeof(unsigned char));
        //FRAMERX.dim = 0;
        //QF.q = (CAMFRAME *)calloc(10, sizeof(CAMFRAME));
        
		a = (unsigned long int)(CAMBUF[3]);
        b = (unsigned long int)(CAMBUF[4]);
        c = (unsigned long int)(CAMBUF[5]);
        IMAGESIZE = a + (b<<8) + (c<<16);
     }
     return 1;    
}


/* SW_Error - Read me:
   This function can write the software error number in CAMSTATUS.   
*/
void Cam_SWError(unsigned int errtype)
{
	CAMSTATUS = CAMSTATUS & (~(ERROR_NUM<<ERROR_FLAG));     /* Clear ERROR flag */
    CAMSTATUS = CAMSTATUS + errtype;           				// SW_error
    return;         
}


/* 	unsigned char Cam_ReadJPEGFrame(unsigned int ind):
   	This function checks the consistency of each received JPEG package. In particular the code analyzes 
	the field ID, the length of the package and the final field called Verify Code.
*/
unsigned char Cam_ReadJPEGFrame(unsigned int ind)
{
//    unsigned char xret;
    unsigned int id,size,sum=0,i,vcode,a,b;
    
    //xret = Cam_Readq(&QF, &FRAMETX, QF.size, 0);
    a = FRAME[0];
    b = FRAME[1];
    id = a + (b<<8);
    if(id!=ind)
    {
        Cam_SWError(CAM_ERR_SW_ERROR10);
        return 0;
    }
    a = FRAME[2];
    b = FRAME[3];
    size = a + (b<<8);
    if(size!= (JPEGDIM-6))
    {
        Cam_SWError(CAM_ERR_SW_ERROR11);
        return 0;	
    }
	

    for(i=0;i<size+4;i++)
    {
        sum += (FRAME[i]);
    }
    vcode = FRAME[JPEGDIM-2];
    if((sum & 0x00FF) != vcode) 
    {
        Cam_SWError(CAM_ERR_SW_ERROR12);
        return 0;
    }
    return 1;
}

/* Cam_Sleep - Read me:
   This function can create a delay in the sequence managed by TIMER89 interrupt.
*/
void Cam_Sleep(unsigned long int xTime)
{      
    //Cam_CloseTimer();
    //TMR9  = 0;             /* Reset Timer9 to 0x0000 */
    //PR9   = stbTime;       /* assigning value to Timer period register */

	T8CONbits.TON = 0;       /* Disable Timer 8 */
	Cam_WriteTimer89(0);
	PR8 = xTime;           /* assigning Period to PR8 */
    PR9 = xTime>>16;       /* Period to  PR9Register */
    T8CONbits.T32 = 1;
	T8CONbits.TON= 1;                  /* Turn on Timer89 */ 

    CAMSTATUS = CAMSTATUS | (WAIT_NUM<<WAIT_FLAG);     /* Set timer flag */
     
    while( (CAMSTATUS>>WAIT_FLAG)&WAIT_NUM );
    //Cam_CloseTimer89();
	T8CONbits.TON = 0;  /* Disable Timer 8 */
     
    return;    
}


void Cam_SendInfo(void)
{
    char camst[14];
	while(BusyUART1());
//	camst[0] = 0xAA;
//    camst[1] = 0xBB;
//    camst[2] = 0xCC;
//	camst[3] = (CAMCONFIG>>8) & 0x00FF;
//    camst[4] = CAMCONFIG & 0x00FF;
//    camst[5] = (CAMFRAMESIZE>>8) & 0x00FF;
//    camst[6] = CAMFRAMESIZE & 0x00FF;
//    camst[7] = (CAMDROPFRAMES>>8) & 0x00FF;
//    camst[8] = CAMDROPFRAMES & 0x00FF;
//    camst[9] = (CAMSTATUS>>8) & 0x00FF;
//    camst[10] = CAMSTATUS & 0x00FF;
	camst[0] = 0xAA;
    camst[1] = 0xBB;
    camst[2] = 0xCC;
	camst[3] = (IMAGESIZE>>16) & 0x00FF;
    camst[4] = (IMAGESIZE>>8) & 0x00FF;
    camst[5] = IMAGESIZE & 0x00FF;
    camst[6] = (BCOUNTER>>16) & 0x00FF;
    camst[7] = (BCOUNTER>>8) & 0x00FF;
	camst[8] = BCOUNTER & 0x00FF;
    camst[9] = (CAMSTATUS>>8) & 0x00FF;
    camst[10] = CAMSTATUS & 0x00FF;
	Send_by_UART1((unsigned int *)camst,11); // Characters transimitted in hex.
	while(BusyUART1());
	return;
}
/*************************************************************************************************************/



